@@ -252,10 +252,13 @@ getClassDef <-
         if(is.character(where)) {
             package <- where
         }
-	if(isTRUE(nzchar(package))) {
-	    whereP <- .requirePackage(package)
-	    value <- get0(cname, whereP, inherits = inherits) # NULL if not existing
-	}
+
+        if(isTRUE(nzchar(package))) {
+	    package <- .requirePackage(package)
+        }
+        if (is.environment(package)) {
+            value <- get0(cname, package, inherits = inherits)
+        }
 	if(is.null(value))
 	    value <- get0(cname, where, inherits = inherits) # NULL if not existing
     }
@@ -533,14 +536,14 @@ validObject <- function(object, test = FALSE, complete = FALSE)
 	    break
 	}
 	validityMethod <- superDef@validity
-	if(is(validityMethod, "function")) {
+	if(is.function(validityMethod)) {
 	    errors <- c(errors, anyStrings(validityMethod(as(object, superClass))))
 	    if(length(errors))
 		break
 	}
     }
     validityMethod <- classDef@validity
-    if(length(errors) == 0L && is(validityMethod, "function")) {
+    if(length(errors) == 0L && is.function(validityMethod)) {
 	errors <- c(errors, anyStrings(validityMethod(object)))
     }
     if(length(errors)) {
@@ -569,7 +572,7 @@ setValidity <- function(Class, method, where = topenv(parent.frame())) {
     }
     method <- .makeValidityMethod(Class, method)
     if(is.null(method) ||
-       (is(method, "function") && length(formalArgs(method)) == 1L))
+       (is.function(method) && length(formalArgs(method)) == 1L))
 	ClassDef@validity <- method
     else
 	stop("validity method must be NULL or a function of one argument")
