@@ -1,7 +1,7 @@
 #  File src/library/stats/R/family.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2019 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -356,7 +356,7 @@ binomial <- function (link = "logit")
         if (!is.null(m <- object$model)) {
             y <- model.response(m)
             if(is.factor(y)) {
-                ## ignote weights
+                ## ignore weights
                 yy <- factor(1+rbinom(ntot, size = 1, prob = ftd),
                              labels = levels(y))
                 split(yy, rep(seq_len(nsim), each = n))
@@ -570,9 +570,17 @@ quasi <- function (link = "identity", variance = "constant")
         stats <- link
         linktemp <- if(!is.null(stats$name)) stats$name else deparse(linktemp)
     }
-    vtemp <- substitute(variance)
+    maybeV <- is.character(vtemp <- substitute(variance)) ||
+        (is.symbol(vtemp) && (vtemp == quote(mu) || vtemp == quote(constant))) ||
+        (is.call(vtemp) &&
+         (length(vtemp) == 2L && vtemp == quote(mu(1-mu))) ||
+         (length(vtemp) == 3L && (vtemp == quote(mu^2)     ||
+                                  vtemp == quote(mu^3))))
+    if(!maybeV && (is.list(variance) && !anyNA(match(c("varfun", "validmu"), names(variance)))))
+        variance_nm <- NA
+    else {
     if (!is.character(vtemp)) vtemp <- deparse(vtemp)
-    variance_nm <- vtemp
+    variance_nm <- vtemp <- gsub(" ", "", vtemp)
     switch(vtemp,
            "constant" = {
                varfun <- function(mu) rep.int(1, length(mu))
@@ -614,10 +622,10 @@ quasi <- function (link = "identity", variance = "constant")
            },
            variance_nm <- NA
            )# end switch(.)
-
+    }
     if(is.na(variance_nm)) {
         if(is.character(variance))
-            stop(gettextf('\'variance\' "%s" is invalid: possible values are "mu(1-mu)", "mu", "mu^2", "mu^3" and "constant"', variance_nm), domain = NA)
+            stop(gettextf('\'variance\' "%s" is invalid: possible values are "mu(1-mu)", "mu", "mu^2", "mu^3" and "constant"', vtemp), domain = NA)
         ## so we really meant the object.
         varfun <- variance$varfun
         validmu <- variance$validmu
