@@ -1,7 +1,7 @@
 #  File src/library/utils/R/str.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2017 The R Core Team
+#  Copyright (C) 1995-2018 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -162,8 +162,17 @@ str.default <-
         vec.len <- 0
     }
 
-    ## x: character
-    maybe_truncate <- function(x, nx = nchar(x, type="w"), S = "\"", ch = "| __truncated__")
+    nchar.w <- function(x) nchar(x, type="w", allowNA=TRUE)
+    ncharN  <- function(x) {
+	r <- nchar(x, type="w", allowNA=TRUE)
+	if(anyNA(r)) {
+	    iN <- is.na(r)
+	    r[iN] <- nchar(x[iN], type="bytes")
+	}
+	r
+    }
+    ## x: character ; using "global" 'nchar.max'
+    maybe_truncate <- function(x, nx = nchar.w(x), S = "\"", ch = "| __truncated__")
     {
 	ok <- if(anyNA(nx)) !is.na(nx) else TRUE
 	if(any(lrg <- ok & nx > nchar.max)) {
@@ -236,6 +245,20 @@ str.default <-
     if (is.null(object))
 	cat(" NULL\n")
     else if(S4) {
+	trygetSlots <- function(x, nms) {
+	    r <- tryCatch(sapply(nms, methods::slot, object=x, simplify = FALSE),
+			  error = conditionMessage)
+	    if(is.list(r))
+		r
+	    else {
+		warning("Not a validObject(): ", r, call.=FALSE) # instead of error
+		r <- attributes(x) ## "FIXME" low-level assumption about S4 slots
+		r <- r[names(r) != "class"]
+		dp <- list(methods::getDataPart(x, NULL.for.none=TRUE))
+		if(!is.null(dp)) names(dp) <- methods:::.dataSlot(nms)
+		c(r, dp)
+	    }
+	}
 	if(methods::is(object,"envRefClass")) {
 	    cld <- tryCatch(object$getClass(), error=function(e)e)
 	    if(inherits(cld, "error")) {
@@ -266,20 +289,19 @@ str.default <-
 		else cat("\n")
 	    }
 	    if(length(sNms <- sNms[sNms != ".xData"])) {
-		sls <- sapply(sNms, methods::slot,
-			      object=object, simplify = FALSE)
 		cat(" and ", n.of(length(sNms), "slot"), "\n", sep="")
+		sls <- trygetSlots(object, sNms)
 		strSub(sls, comp.str = "@ ", no.list=TRUE, give.length=give.length,
 		       indent.str = paste(indent.str,".."), nest.lev = nest.lev + 1)
 	    }
 	    else if(lo == 0) cat(".\n")
 	}
 	else { ## S4 non-envRefClass
-	    a <- sapply(methods::.slotNames(object), methods::slot,
-			object=object, simplify = FALSE)
+	    sNms <- methods::.slotNames(object)
 	    cat("Formal class", " '", paste(cl, collapse = "', '"),
 		"' [package \"", attr(cl,"package"), "\"] with ",
-		n.of(length(a), "slot"), "\n", sep = "")
+		n.of(length(sNms), "slot"), "\n", sep = "")
+	    a <- trygetSlots(object, sNms)
 	    strSub(a, comp.str = "@ ", no.list=TRUE, give.length=give.length,
 		   indent.str = paste(indent.str,".."), nest.lev = nest.lev + 1)
 	}
@@ -316,7 +338,7 @@ str.default <-
 	    if (is.na(max.level) || nest.lev < max.level) {
 		nam.ob <-
 		    if(is.null(nam.ob <- names(object))) rep.int("", le)
-		    else { ncn <- nchar(nam.ob, type="w")
+		    else { ncn <- nchar.w(nam.ob)
 			   if(anyNA(ncn)) ## slower, but correct:
 			      ncn <- vapply(nam.ob, format.info, 0L)
 			   format(nam.ob, width = max(ncn), justify="left")
@@ -351,7 +373,7 @@ str.default <-
 	    object <- unclass(object)
 	    if(nl) {
 		## as from 2.1.0, quotes are included ==> '-2':
-		lenl <- cumsum(3 + (nchar(lev.att, type="w") - 2))# level space
+		lenl <- cumsum(3 + (ncharN(lev.att) - 2))# level space
 		ml <- if(n.l <= 1 || lenl[n.l] <= 13)
 		    n.l else which.max(lenl > 13)
 		lev.att <- maybe_truncate(lev.att[seq_len(ml)])
@@ -548,18 +570,21 @@ str.default <-
 	    ## FIXME: need combined  encode.and.trim.string(object, m)  with O(m) !
 	    encObj <- tryCatch(strtrim(object, trimWidth), error=function(e) NULL)
 	    encObj <-
-		if(is.null(encObj)) # must first encodeString() before we can trim
-		    strtrim(encodeString(object, quote= '"', na.encode= FALSE),
-			    trimWidth)
+		if(is.null(encObj)) { # must first encodeString() before we can trim
+		    e <- encodeString(object, quote= '"', na.encode= FALSE)
+		    r <- tryCatch(strtrim(e, trimWidth), error=function(.) NULL)
+		    ## What else can we try?
+		    if(is.null(r)) e else r
+		}
 		else
 		    encodeString(encObj, quote= '"', na.encode= FALSE)
 	    if(le > 0) ## truncate if LONG char:
 		encObj <- maybe_truncate(encObj)
 	    v.len <-
 		if(missing(vec.len)) {
-		    max(1,sum(cumsum(1 + if(le>0) nchar(encObj, type="w") else 0) <
+		    max(1,sum(cumsum(1 + if(le>0) ncharN(encObj) else 0) <
 			      width - (4 + 5*nest.lev + nchar(str1, type="w"))))
-		}		      # '5*ne..' above is fudge factor
+		}	 	      # '5*ne..' above is fudge factor
 		else round(v.len)
 	    ile <- min(le, v.len)
 	    if(ile >= 1)
@@ -601,8 +626,7 @@ ls.str <-
     if(missing(envir)) ## [for "lazy" reasons, this fails as default]
         envir <- as.environment(pos)
     nms <- ls(name, envir = envir, all.names=all.names, pattern=pattern)
-    r <- unlist(lapply(nms, function(n)
-                       exists(n, envir= envir, mode= mode, inherits=FALSE)))
+    r <- vapply(nms, exists, NA, envir=envir, mode=mode, inherits=FALSE)
     structure(nms[r], envir = envir, mode = mode, class = "ls_str")
 }
 
