@@ -1,8 +1,8 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2017   The R Core Team
+ *  Copyright (C) 1998-2018   The R Core Team
  *  Copyright (C) 2002-2015   The R Foundation
+ *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -242,7 +242,7 @@ SEXP alloc3DArray(SEXPTYPE mode, int nrow, int ncol, int nface)
 	error(_("negative extents to 3D array"));
 #ifndef LONG_VECTOR_SUPPORT
     if ((double)nrow * (double)ncol * (double)nface > INT_MAX)
-	error(_("'alloc3Darray': too many elements specified"));
+	error(_("'alloc3DArray': too many elements specified"));
 #endif
     n = ((R_xlen_t) nrow) * ncol * nface;
     PROTECT(s = allocVector(mode, n));
@@ -261,11 +261,13 @@ SEXP allocArray(SEXPTYPE mode, SEXP dims)
     SEXP array;
     int i;
     R_xlen_t n = 1;
+#ifndef LONG_VECTOR_SUPPORT
     double dn = 1;
+#endif
 
     for (i = 0; i < LENGTH(dims); i++) {
-	dn *= INTEGER(dims)[i];
 #ifndef LONG_VECTOR_SUPPORT
+	dn *= INTEGER(dims)[i];
 	if(dn > INT_MAX)
 	    error(_("'allocArray': too many elements specified by 'dims'"));
 #endif
@@ -416,7 +418,7 @@ SEXP attribute_hidden do_drop(SEXP call, SEXP op, SEXP args, SEXP rho)
 	for (i = 0; i < n; i++)
 	    if (INTEGER(xdims)[i] == 1) shorten = 1;
 	if (shorten) {
-	    if (MAYBE_REFERENCED(x)) x = duplicate(x);
+	    if (MAYBE_REFERENCED(x)) x = R_duplicate_attr(x);
 	    x = DropDims(x);
 	}
     }
@@ -482,8 +484,13 @@ R_xlen_t attribute_hidden dispatch_xlength(SEXP x, SEXP call, SEXP rho) {
 
 // auxiliary for do_lengths_*(), i.e., R's lengths()
 static R_xlen_t getElementLength(SEXP x, R_xlen_t i, SEXP call, SEXP rho) {
-    SEXP x_elt = dispatch_subset2(x, i, call, rho);
-    return(dispatch_xlength(x_elt, call, rho));
+    SEXP x_elt;
+    R_xlen_t ans;
+
+    PROTECT(x_elt = dispatch_subset2(x, i, call, rho));
+    ans = dispatch_xlength(x_elt, call, rho);
+    UNPROTECT(1); /* x_elt */
+    return ans;
 }
 
 #ifdef LONG_VECTOR_SUPPORT
@@ -564,31 +571,32 @@ SEXP attribute_hidden do_lengths(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 SEXP attribute_hidden do_rowscols(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP x, ans;
-    int i, j, nr, nc;
-
     checkArity(op, args);
-    /* This is the dimensions vector */
-    x = CAR(args);
-    if (!isInteger(x) || LENGTH(x) != 2)
+    SEXP dim = CAR(args);
+    int nprot = 0;
+    if (!isInteger(dim)) {
+	PROTECT(dim = coerceVector(dim, INTSXP)); nprot++;
+    }
+    if (LENGTH(dim) != 2)
 	error(_("a matrix-like object is required as argument to '%s'"),
 	      (PRIMVAL(op) == 2) ? "col" : "row");
 
-    nr = INTEGER(x)[0];
-    nc = INTEGER(x)[1];
+    int nr = INTEGER(dim)[0],
+	nc = INTEGER(dim)[1];
+    if(nprot) UNPROTECT(nprot);
 
-    ans = allocMatrix(INTSXP, nr, nc);
+    SEXP ans = allocMatrix(INTSXP, nr, nc);
 
     R_xlen_t NR = nr;
     switch (PRIMVAL(op)) {
-    case 1:
-	for (i = 0; i < nr; i++)
-	    for (j = 0; j < nc; j++)
+    case 1: // row() & .row()
+	for (int i = 0; i < nr; i++)
+	    for (int j = 0; j < nc; j++)
 		INTEGER(ans)[i + j * NR] = i + 1;
 	break;
-    case 2:
-	for (i = 0; i < nr; i++)
-	    for (j = 0; j < nc; j++)
+    case 2: // col() & .col()
+	for (int i = 0; i < nr; i++)
+	    for (int j = 0; j < nc; j++)
 		INTEGER(ans)[i + j * NR] = j + 1;
 	break;
     }
@@ -2102,11 +2110,13 @@ SEXP attribute_hidden do_array(SEXP call, SEXP op, SEXP args, SEXP rho)
 	else
 	    for (i = 0; i < nans; i++) RAW(ans)[i] = 0;
 	break;
-    /* Rest are already initialized */
     case STRSXP:
 	if (nans && lendat)
 	    xcopyStringWithRecycle(ans, vals, 0, nans, lendat);
+	else
+	    for (i = 0; i < nans; i++) SET_STRING_ELT(ans, i, NA_STRING);
 	break;
+    /* Rest are already initialized */
     case VECSXP:
     case EXPRSXP:
 #ifdef SWITCH_TO_REFCNT
