@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000-2018	The R Core Team.
+ *  Copyright (C) 2000-2019	The R Core Team.
  *  Copyright (C) 1995-1998	Robert Gentleman and Ross Ihaka.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -1017,7 +1017,12 @@ void attribute_hidden PrintValueEnv(SEXP s, SEXP env)
 
     R_PrintData data;
     PrintInit(&data, env);
-    PrintDispatch(s, &data);
+    if (isFunction(s))
+	/* printed via print() -> print.function() in order to allow user-defined
+	   print.function() methods to also work in auto-printing: */
+        PrintObject(s, &data);
+    else
+        PrintDispatch(s, &data);
 
     UNPROTECT(1);
 }
@@ -1050,11 +1055,12 @@ void attribute_hidden CustomPrintValue(SEXP s, SEXP env)
 
 
 /* xxxpr are mostly for S compatibility (as mentioned in V&R).
-   The actual interfaces are now in xxxpr.f
+   The Fortran interfaces are in xxxpr.f and call these.
+    They are always called with *nchar >= 0.
  */
 
 attribute_hidden
-int F77_NAME(dblep0) (const char *label, int *nchar, double *data, int *ndata)
+void F77_NAME(dblep0) (const char *label, int *nchar, double *data, int *ndata)
 {
     int k, nc = *nchar;
 
@@ -1068,11 +1074,10 @@ int F77_NAME(dblep0) (const char *label, int *nchar, double *data, int *ndata)
 	Rprintf("\n");
     }
     if(*ndata > 0) printRealVector(data, *ndata, 1);
-    return(0);
 }
 
 attribute_hidden
-int F77_NAME(intpr0) (const char *label, int *nchar, int *data, int *ndata)
+void F77_NAME(intpr0) (const char *label, int *nchar, int *data, int *ndata)
 {
     int k, nc = *nchar;
 
@@ -1086,11 +1091,10 @@ int F77_NAME(intpr0) (const char *label, int *nchar, int *data, int *ndata)
 	Rprintf("\n");
     }
     if(*ndata > 0) printIntegerVector(data, *ndata, 1);
-    return(0);
 }
 
 attribute_hidden
-int F77_NAME(realp0) (const char *label, int *nchar, float *data, int *ndata)
+void F77_NAME(realp0) (const char *label, int *nchar, float *data, int *ndata)
 {
     int k, nc = *nchar, nd = *ndata;
     double *ddata;
@@ -1112,14 +1116,16 @@ int F77_NAME(realp0) (const char *label, int *nchar, float *data, int *ndata)
 	printRealVector(ddata, nd, 1);
 	free(ddata);
     }
-    return(0);
 }
 
 /* Fortran-callable error routine for lapack */
 
 void NORET F77_NAME(xerbla)(const char *srname, int *info)
 {
-   /* srname is not null-terminated.  It should be 6 characters. */
+    /* srname is not null-terminated.  It will be 6 characters for
+      mainstream BLAS/LAPACK routines (for those with < 6 the name
+      is right space-padded), and > 6 for recentish additions from
+      LAPACK, 7 for a few used with R ). */
     char buf[7];
     strncpy(buf, srname, 6);
     buf[6] = '\0';
